/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function generateFile(dir, filename){
	var file = dir.clone();
	file.append(filename);
	if(!file.exists())
		file.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0666);
	return file;
}
 
function getFileObject(){
	var file = null;
	if(Prefs.getBool("indexPathDefault")){
		file = Components.classes["@mozilla.org/file/directory_service;1"]
			.getService(Components.interfaces.nsIProperties)
			.get("ProfD", Components.interfaces.nsIFile);
	}else {
		file = Prefs.getFile("indexPath");
	}
	return file;
} 

/*
	Returns the build directory.
 */
function getBuildDir(){
	var file = getFileObject();

	file.append("cl_index");
	file.append("build");
	
	if(!file.exists() || !file.isDirectory()) 
		file.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);

	return file.clone();
}

/*
	Returns the search directory.
*/
function getSearchDir(){
	var file = getFileObject();

	file.append("cl_index");
	file.append("search");
	
	if(!file.exists() || !file.isDirectory()) 
		file.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);

	return file.clone();
}

/*
	Moves all files from the build to the search directory.
*/
function move(bdir, sdir){
	var entries = bdir.directoryEntries;
	while(entries.hasMoreElements()){
	var file = entries.getNext().QueryInterface(Components.interfaces.nsIFile);
		if(!file.isDirectory()){
			// move file without renaming
			file.moveTo(sdir, file.leafName);
		}
	}
}

/*
	Clears search and build directory.
*/
function clearDirectories(){
	clear(getSearchDir());
	clear(getBuildDir());
}

/*
	Clears a given directory.
*/
function clear(dir){
	var entries = dir.directoryEntries;
	while(entries.hasMoreElements()){
		var file = entries.getNext().QueryInterface(Components.interfaces.nsIFile);
		if(file.isDirectory()){
			file.remove(true);
		}else {
			file.remove(false);
		}
	}
}

/*
	Returns the size (bytes)  of a given directory.
*/
function getDirectoryInfo(dir){
	var size = 0;
	var entries = dir.directoryEntries;
	
	while(entries.hasMoreElements()){
		var file = entries.getNext().QueryInterface(Components.interfaces.nsIFile);
		if(file.isDirectory()){
			size += getDirectoryInfo(file)
		}else {
			size += file.fileSize;
		}
	}
	return size;
}

/*
	Formats a given byte number to kb or mb.
*/
function formatBytes(filesize){

	if (filesize >= 1073741824) {
		filesize = number_format(filesize / 1073741824, 2, '.', '') + ' Gb';
	} else {
		if (filesize >= 1048576) {
			filesize = number_format(filesize / 1048576, 2, '.', '') + ' Mb';
		} else {
			if (filesize >= 1024) {
				filesize = number_format(filesize / 1024, 0) + ' Kb';
			} else {
				filesize = number_format(filesize, 0) + ' bytes';
			}
		}
	}
	return filesize;
}

function number_format( number, decimals, dec_point, thousands_sep ) {
    // http://kevin.vanzonneveld.net
    // +   original by: Jonas Raoni Soares Silva (http://www.jsfromhell.com)
    // +   improved by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
    // +     bugfix by: Michael White (http://crestidg.com)
    // +     bugfix by: Benjamin Lupton
    // +     bugfix by: Allan Jensen (http://www.winternet.no)
    // +    revised by: Jonas Raoni Soares Silva (http://www.jsfromhell.com)    
    // *     example 1: number_format(1234.5678, 2, '.', '');
    // *     returns 1: 1234.57     
 
    var n = number, c = isNaN(decimals = Math.abs(decimals)) ? 2 : decimals;
    var d = dec_point == undefined ? "," : dec_point;
    var t = thousands_sep == undefined ? "." : thousands_sep, s = n < 0 ? "-" : "";
    var i = parseInt(n = Math.abs(+n || 0).toFixed(c)) + "", j = (j = i.length) > 3 ? j % 3 : 0;
    
    return s + (j ? i.substr(0, j) + t : "") + i.substr(j).replace(/(\d{3})(?=\d)/g, "$1" + t) + (c ? d + Math.abs(n - i).toFixed(c).slice(2) : "");
}

function writeLineToFile(file, value){
	if(file){
		var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
						.createInstance(Components.interfaces.nsIFileOutputStream);

		// use 0x02 | 0x10 to open file for appending.
		foStream.init(file, 0x02 | 0x08 | 0x10, 0666, 0); // write, create, truncate

		foStream.write(value, value.length);
		foStream.close();
	}
}

/*
	Reads the given file line by line and returns a extended hashmap
	with line as key and the line number as value.
*/
function readFileLineByLine(file){
	// open an input stream from file
	var istream = Components.classes["@mozilla.org/network/file-input-stream;1"].
                        createInstance(Components.interfaces.nsIFileInputStream);
	istream.init(file, 0x01, 0444, 0);
	istream.QueryInterface(Components.interfaces.nsILineInputStream);

	// read lines into array
	var line = {}, lines = new ExtendedHash(), hasmore, count = 0;
	hasmore = istream.readLine(line);
	
	while(hasmore){
		lines.put(line.value, count); 
		hasmore = istream.readLine(line);
		count++;
	}

	istream.close();

	return lines;
}

function readFile(file){
	var data = "";
	var fstream = Components.classes["@mozilla.org/network/file-input-stream;1"]
	                        .createInstance(Components.interfaces.nsIFileInputStream);
	var sstream = Components.classes["@mozilla.org/scriptableinputstream;1"]
	                        .createInstance(Components.interfaces.nsIScriptableInputStream);
	fstream.init(file, -1, 0, 0);
	sstream.init(fstream); 

	var str = sstream.read(4096);
	while (str.length > 0) {
	  data += str;
	  str = sstream.read(4096);
	}

	sstream.close();
	fstream.close();
	return data;
}

var InstallRDF = {

	version : "",
	name : "",
	description : "",
	creator : "",
	homepageURL : "",
	appID : "maik.gollnick@gmx.de",

	init : function(){
		InstallRDF.parse();
	},
	
	parse : function(){
		var em = Components.classes["@mozilla.org/extensions/manager;1"]
				.getService(Components.interfaces.nsIExtensionManager);
		var rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"]
						.getService(Components.interfaces.nsIRDFService);
		var rdfLiteral = Components.interfaces.nsIRDFLiteral;
		
		var file = em.getInstallLocation(InstallRDF.appID).getItemFile(InstallRDF.appID, "install.rdf");

		var p = "file:///" + file.path.replace(/\\/g, "/");
		var fileDS = rdfService.GetDataSourceBlocking(p);
				
		var e = fileDS.GetAllResources();
		while(e.hasMoreElements()) {
	    	var r =  e.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
			if(r.Value == "urn:mozilla:install-manifest"){

				var t0 = fileDS.GetTarget(r, rdfService.GetResource("http://www.mozilla.org/2004/em-rdf#version"), true);
				var t1 = fileDS.GetTarget(r, rdfService.GetResource("http://www.mozilla.org/2004/em-rdf#name"), true);
				var t2 = fileDS.GetTarget(r, rdfService.GetResource("http://www.mozilla.org/2004/em-rdf#description"), true);
				var t3 = fileDS.GetTarget(r, rdfService.GetResource("http://www.mozilla.org/2004/em-rdf#creator"), true);
				var t4 = fileDS.GetTarget(r, rdfService.GetResource("http://www.mozilla.org/2004/em-rdf#homepageURL"), true);

				if(t0) InstallRDF.version = " v." + t0.QueryInterface(rdfLiteral).Value;
				if(t1) InstallRDF.name = t1.QueryInterface(rdfLiteral).Value;
				if(t2) InstallRDF.description = t2.QueryInterface(rdfLiteral).Value;
				if(t3) InstallRDF.creator = t3.QueryInterface(rdfLiteral).Value;
				if(t4) InstallRDF.homepageURL = t4.QueryInterface(rdfLiteral).Value;
			}
		}
	}
};

InstallRDF.init();