/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
const AUTOMATIC = 0;
const MANUAL = 1;

const DELAY_ONE_WEEK = 604800000;
const DELAY_TWO_WEEK = 1209600000;
const DELAY_THREE_WEEK = 1814400000;
const DELAY_FOUR_WEEK = 2419200000;

/*
	Basic indexer that wraps the clucene component into the js world.	
*/
var Index = {

	path : "",
	delay : DELAY_ONE_WEEK, //default delay one week (in millis)
	mode : MANUAL, //default mode
	filter : null,
	crawler : null,
	
	indexWriter : null,
	indexReader : null,

	//TODO: implement _init()
	init : function(){
		var instWriter = Components.classes["@bookmarktools.mozdev.org/search/indexWriter;1"].createInstance();
		this.indexWriter = instWriter.QueryInterface(Components.interfaces.ICLWriter);
			
		var instReader = Components.classes["@bookmarktools.mozdev.org/search/indexReader;1"].createInstance();
		this.indexReader = instReader.QueryInterface(Components.interfaces.ICLReader);

		this.indexReader.setPath(getSearchDir().path);
		this.setMode(Prefs.getInt("buildType"));
		
		switch(Prefs.getInt("indexDelay")){
			case 1: this.delay = DELAY_ONE_WEEK; break;
			case 2: this.delay = DELAY_TWO_WEEK; break;
			case 3: this.delay = DELAY_THREE_WEEK; break;
			case 4: this.delay = DELAY_FOUR_WEEK; break;
			default: throw 'Error: Unknown delay!';
		}
	},
	
	setMode : function(mode){
		this.mode = mode;
		switch(this.mode){
			case AUTOMATIC: 
				this.path = getBuildDir().path;
				break;
			case MANUAL:
				this.path = getSearchDir().path;
				break;
			default: throw 'Error: Unknown build type!';
		}
	
		this.indexWriter.setPath(this.path);
	},
	
	setFilter : function(filter){
		this.filter = filter;
		this.filter.onComplete = this.onComplete;
		this.filter.onDone = this.onDone;
		this.filter.onAbort = this.onAbort;
	},
	
	setCrawler : function(crawler){
		this.crawler = crawler;
		this.crawler.onComplete = this.onComplete;
		this.crawler.onDone = this.onDone;
		this.crawler.onAbort = this.onAbort;
	},
	
	onComplete : function(combinedContent, bookmark){
		Index.indexWriter.appendDocument(
			Book.getURI(bookmark),
			Book.getName(bookmark),
			combinedContent
		);
		if(Index.mode == AUTOMATIC){
			var file = Prefs.getFile('alreadyAddedBookmarks');
			writeLineToFile(file, Book.getURI(bookmark) + String.LF);
		}
	},
	
	onDone : function(){
		if(Index.filter){
			if(Index.filter.view){
				Index.filter.view._init();
				if(Index.mode == MANUAL) startLookup();
			}
			Index.filter._init();
		}
		Index.indexWriter.optimize();
		
		if(Index.mode == AUTOMATIC){
			Prefs.setBool('wasUpdated', true);
			Prefs.setBool('firstRun', true);
		}
			
		var currentTime = new Date().getTime();
		Prefs.setChar('intIndexDate', currentTime);
	},
	
	onAbort : function(){
		if(Index.filter){
			if(Index.filter.view){
				Index.filter.view._init();
				if(Index.mode == MANUAL) startLookup();
			}
			Index.filter._init();
		}
		Index.indexWriter.optimize();
	},
	
	isValidBuildTime : function(){
		if(this.mode == AUTOMATIC){
			var lastBuildTime = 0;
			try {
				lastBuildTime = parseInt(Prefs.getChar('intIndexDate'));
			} catch(e){}
			
			var currentTime = new Date().getTime();
			
			if(isNaN(lastBuildTime)){
				lastBuildTime = 0
			}
	
			if((lastBuildTime + this.delay) <= currentTime){
				return true;
			}
		}
		return false;
	},
	
	// starts building the index
	_build : function(){
		if(this.filter){
			this.filter.go();
		}else {
			if(this.crawler){ //TODO: check -> useful case?
				this.crawler._build();
			}else {
				throw 'Error: No filter available!';
			}
		}
	},

	// finds the pattern and update the given container instance
	_search : function(pattern, container, chunk, chunkSize){
		var max = 0;
		if(this.indexReader.exists()){ //index exists ?
			// validate pattern
			setAlertUser(true);
			//TODO: check and replace???
			var query = !doCheckLuceneQueryValue(pattern) ? "" : pattern;

			if(query.length > 2){

				var startTime = new Date().getTime();
				var array = nsIMutableArray();
				max = this.indexReader.search(query, array, chunk, chunkSize);

				//clear result tree
				container.clear();

				var bookmarks = Book.getResourceHash();
				
				//draw results
				var e = array.enumerate();
				while (e.hasMoreElements()) {
					var resultNode = e.getNext().QueryInterface(Components.interfaces.IResultNode);
					if(bookmarks.contains(resultNode.primaryKey)){
						container.add(
							bookmarks.get(resultNode.primaryKey), 
							resultNode.score
						);
					}
				}
				
			}else {
				container.clear();
			}
		}else {
			throw 'Index is empty!';
		}
		return max;
	},
	
	// delete documents
	_delete : function(nodes){
		if(nodes.length != 0){
			this.indexWriter.deleteDocument(nodes, nodes.length);
		}else {
			// it is possible to delete every document but
			//faster to clear the whole directory
			clear(getSearchDir());
		}
	},

	// return the index as mutable array
	_lookup : function(){
		var array = nsIMutableArray();
		this.indexReader.lookup(array);
		return array;
	},
	
	// if build is running this function will abort the crawler
	_abort : function(){
		if(!this.filter) throw 'Error: No filter available!';
		if(!this.filter.crawler) throw 'Error: No crawler available!';
		
		this.filter.crawler._abort();
	}

};

Index.init();

var Filter = {

	uniqueContent : null,

	countDuplicate : 0,
	countError : 0,
	count : 0,
	max : 0,
	
	crawler : null,
	view : null,
	
	check : false,
	
	logger : null,
	
	init : function(){
		Parser.init();
		this.uniqueContent = [];
		this.logger = new Logger(this, arguments);
		this.logger.init("FilterLog.log");
	},
	
	_init : function(){
		Parser._init();
		this.uniqueContent = null;
		this.countDuplicate = 0;
		this.countError = 0;
		this.count = 0;
		this.max = 0;
		this.check = false;
	},
	
	setCrawler : function(crawler, count, max){
		this.crawler = crawler;
		this.crawler.onComplete = this.filter;
		this.crawler.onError = this.updateError;
		this.crawler.onDone = this.onDone;
		this.crawler.onAbort = this.onAbort;
		
		if(arguments.length == 3){
			this.count = count;
			this.max = max;
		}else {
			this.max = this.crawler.bookmarks.length();
		}
	},
	
	getBookmarksFromFile : function(){
		var file = Prefs.getFile('alreadyAddedBookmarks');
		var alreadyAddedBookmarks = readFileLineByLine(file);
	
		var bookmarks = Book.getResourceHash();
		var notAddedBookmarks = new ExtendedHash();
		
		if(alreadyAddedBookmarks.isEmpty()){
			return {
				currentIndexed : 0,
				allBookmarks : bookmarks.length(),
				remainingBookmarks : bookmarks
			};
		}

		bookmarks.each(function(key, value){
			if(!alreadyAddedBookmarks.contains(key)){
				notAddedBookmarks.put(key, value);
			}
		});
		
		return {
			currentIndexed : alreadyAddedBookmarks.length(),
			allBookmarks : bookmarks.length(),
			remainingBookmarks : notAddedBookmarks
		};
	},
	
	onComplete : function(combinedContent, bookmark){},
	
	onDone : function(){
		if(Filter.view)
			Filter.view._init();
		Filter._init();
	},
	
	onAbort : function(){
		if(Filter.view)
			Filter.view._init();
		Filter._init();
	},
	
	setView : function(view){
		this.view = view;
	},
	
	setCheck : function(check){
		this.check = check;
	},
	
	go : function(){
		if(!this.crawler) throw 'Error: No crawler available!';
		
		//url duplicates loaded one time at startup in automatic build mode
		if((Index.mode == AUTOMATIC && Prefs.getBool('firstRun')) ||
			Index.mode == MANUAL){
			var duplicateURLs = this.crawler.bookmarks.trash();
			duplicateURLs.each(function(elem){
				var uri = Book.getURI(elem);
				Filter.updateDuplicate(uri, uri);
			});
		}
		
		//update counter because automatic indexing
		//is done in 0-n steps and every step produces duplicates
		if(Index.mode == AUTOMATIC)
			this.countDuplicate = Prefs.getBranch('dupBm').length;
	
		this.crawler._build();
	},
	
	//overwrites crawlers onNext and onUpdate
	filter : function(responseText, mimeType, charSet, bookmark){
		var content = String.empty;
		try{
			content = Parser.parse(mimeType, charSet, responseText);
		}catch(e){
			alert(e);
		}

		var uri = Book.getURI(bookmark);
		
		if(!Filter.isDuplicate(content, uri)){
			if(!Filter.check){
				var combinedContent = Filter.combineContent(content, bookmark);
				Filter.onComplete(combinedContent, bookmark);
			}	
		}
	
		Filter.count++;
		if(Filter.view)
			Filter.view.setProgress(Filter.count, Filter.max, uri);
	},
	
	combineContent : function(content, bookmark){
		content += ' ';
		content += Book.getName(bookmark);
		content += ' ';
		content += Book.getURI(bookmark);
		content += ' ';
		content += Book.getDesc(bookmark);
		content += ' ';
		content += Book.getKeys(bookmark);
		return content.trim();
	},
	
	combineWithoutContent : function(bookmark){
		var combined = Book.getName(bookmark);
		combined += ' ';
		combined += Book.getURI(bookmark);
		combined += ' ';
		combined += Book.getDesc(bookmark);
		combined += ' ';
		combined += Book.getKeys(bookmark);
		return combined.trim();
	},
	
	trimContent : function(content){
		var trimed = content.stripScripts();
		trimed = Parser.removeWhiteSpaces(trimed);
		return Parser.removeNumbers(trimed);
	},
	
	isDuplicate : function(content, uri){
		var trimedContent = Filter.trimContent(content);
	
		//if no content is available continue indexing, that
		//appends all bookmark properties in every case
		if(trimedContent.empty()) return false;
		
		if(!Filter.uniqueContent[trimedContent]){
			Filter.uniqueContent[trimedContent] = uri;
			return false;
		}
		
		Filter.updateDuplicate(uri, Filter.uniqueContent[trimedContent]);
		
		return true;
	},

	updateDuplicate : function(uri, equals){
		var json = JSONUtils.toJSON(
			{
				uri: uri,
				equals: equals
			}
		);
		Prefs.setChar('dupBm' + Filter.countDuplicate++, json);
	
		var button = $("DUPLICATES");
		if(button){
			var l = button.label;
			l = l.replace(/\s(.*)/gi, "");
				
			button.label = l + " (" + Filter.countDuplicate + ")";
			button.disabled = Filter.countDuplicate == 0;
		}
	},
	
	updateError : function(status, statusText, bookmark){
		Filter.count++;
		var json = JSONUtils.toJSON(
			{
				uri: Book.getURI(bookmark),
				status: status,
				statusText: statusText
			}
		);
		Prefs.setChar('invBm' + Filter.countError++, json);
	
		var error_tree = $('ERRORTREE');
		var other_tree = $('OTHERTREE');
		
		if((error_tree || other_tree) && typeof updateView != 'undefined')
			updateView(status, statusText, bookmark, Prefs.getInt('errorView'));
		
		//if error type equals 'OTHER' the bookmark is valid but no content available
		//-> in this case only the bookmark properties combined for index document
		if(status <= -400){
			if(!Filter.check){
				var combined = Filter.combineWithoutContent(bookmark);
				Filter.onComplete(combined, bookmark);
			}	
		}
	}

};

var IndexLock = {

	acquire : function(){
		var key = new Date().getTime();
		var lock = Prefs.getChar('indexLock');

		if(lock.empty()){
			Prefs.setChar('indexLock', key);
			return key;
		}
		
		return -1;
	},
	
	release : function(key){
		var lock = Prefs.getChar('indexLock');
		if(key == lock){
			Prefs.setChar('indexLock', String.empty);
		}
	}

};

var View = {

	doc : null,
	win : null,
	panel : null,
	
	init : function(){
		var w = nsIWindowMediator('bookmarktools:manual:indexing');
		View.win = w[0];
		View.doc = w[0].document;
		View.panel = {
			label: View.doc.getElementById('PROGRESS.INFO'),
			progressLabel: View.doc.getElementById('PROGRESS.LABEL'),
			progressBar: View.doc.getElementById('PROGRESS.BAR')
		};
	},
	
	_init : function(){
		View.win.close();
		View.win = null;
		View.doc = null;
		View.panel = null;
	},

	percent : function(i, j){
		return Math.floor((i / j) * 100);
	},
	
	formatURI : function(i, j, uri, maxLength){
		var range = '[' + i + '/' + j + '] ';
		return range + uri.truncate(maxLength, ' [..]');
	},
	
	formatProgress : function(percent){
		return percent + '%';
	},
	
	setProgress : function(i, j, uri){
		View.panel.label.value = View.formatURI(i, j, uri, 30);
		var val = View.percent(i, j);
		View.panel.progressLabel.value = View.formatProgress(val);
		View.panel.progressBar.value = val;
	}

};

var BrowserView = {
	
	doc : null,
	win : null,
	panel : null,
	
	init : function(){
		BrowserView.win = nsIBrowserWindow();
		BrowserView.doc = BrowserView.win.document;
		BrowserView.panel = {
			progressLabel: BrowserView.doc.getElementById('AUTOPROGRESS.LABEL'),
			progressBar: BrowserView.doc.getElementById('AUTOPROGRESS.BAR')
		};
		$('autoProgressOverlay').hidden = false;
	},
	
	_init : function(){
		$('autoProgressOverlay').hidden = true;
	},
	
	percent : function(i, j){
		return Math.floor((i / j) * 100);
	},
	
	formatProgress : function(percent){
		return percent + '%';
	},
	
	setProgress : function(i, j){
		var val = BrowserView.percent(i, j);
		BrowserView.panel.progressLabel.value = BrowserView.formatProgress(val);
		BrowserView.panel.progressBar.value = val;
	}
};
