/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

String.empty = '';
String.LF = '\n';
String.SPACE = ' ';
String.TAB = '\t';

String.prototype.trim = function(){
	var start = -1, end = this.length;  
	while(this.charCodeAt(--end) < 33);  
	while(this.charCodeAt(++start) < 33);  
	return this.slice(start, end+1); 
};

String.prototype.setCharAt = function(i, c){
	if(i > this.length - 1 || i < 0) return this;
	return this.substr(0, i) + c + this.substr(i + 1);
};

String.prototype.equalsIgnoreCase = function(str){
	var str0 = this.toLowerCase();
	var str1 = str.toLowerCase();
	return str0 == str1;
};

String.prototype.contains = function(str){
	return this.indexOf(str) != -1;
};

String.prototype.nsiURI = function(){
	var ioService = Components.classes["@mozilla.org/network/io-service;1"]
				.getService(Components.interfaces.nsIIOService);
	return ioService.newURI(this, null, null);
};

Number.prototype.isPositive = function(){
	return this >= 0;
};

Number.prototype.isNegative = function(){
	return this <= 0;
};

Number.prototype.isZero = function(){
	return this == 0;
};

Array.prototype.isEmpty = function(){
	return this.length == 0;
};

Array.prototype.isNotEmpty = function(){
	return this.length != 0;
};

var JSONUtils = {
	toJSON : function(object){
		return Object.toJSON(object);
	},
	
	fromJSON : function(json){
		return eval('(' + json + ')');
	}
};

function nsIMutableArray(){
	return Components.classes["@mozilla.org/array;1"]
		.createInstance(Components.interfaces.nsIMutableArray);
}

function nsIWindowMediator(type){
	var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
			.getService(Components.interfaces.nsIWindowMediator);
	
	var w = [];
	var e = wm.getEnumerator(type);
	
	while(e.hasMoreElements())
		w.push(e.getNext());
	return w;
}

function nsIBrowserWindow(){
	var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
			.getService(Components.interfaces.nsIWindowMediator);
	var mainWindow = wm.getMostRecentWindow("navigator:browser");
	//var browser = mainWindow.getBrowser();
	return mainWindow;
}

var Prefs = {

	root : 'bookmarkTools',

	instance : function(){
		return Components.classes["@mozilla.org/preferences-service;1"]
			.getService(Components.interfaces.nsIPrefBranch);
	},
	
	service : function(){
		return Components.classes["@mozilla.org/preferences-service;1"]
			.getService(Components.interfaces.nsIPrefService);
	},
	
	getFile : function(key){
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                      getService(Components.interfaces.nsIPrefService).
                      getBranch("bookmarkTools.");
		return prefs.getComplexValue(key, Components.interfaces.nsILocalFile);
	},
	
	getBool : function(key){
		var inst = Prefs.instance();
		return inst.getBoolPref(Prefs.root + '.' + key);
	},
	
	getChar : function(key){
		var inst = Prefs.instance();
		return inst.getCharPref(Prefs.root + '.' + key);
	},
	
	getInt : function(key){
		var inst = Prefs.instance();
		return inst.getIntPref(Prefs.root + '.' + key);
	},
	
	setFile : function(key, file){
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                      getService(Components.interfaces.nsIPrefService).
                      getBranch("bookmarkTools.");
		prefs.setComplexValue(key, Components.interfaces.nsILocalFile, file);
	},

	setBool : function(key, val){
		var inst = Prefs.instance();
		inst.setBoolPref(Prefs.root + '.' + key, val);
	},
	
	setChar : function(key, val){
		var inst = Prefs.instance();
		inst.setCharPref(Prefs.root + '.' + key, val);
	},
	
	setInt : function(key, val){
		var inst = Prefs.instance();
		inst.setIntPref(Prefs.root + '.' + key, val);
	},
	
	remove : function(key){
		var inst = Prefs.instance();
		inst.clearUserPref(Prefs.root + '.' + key);
	},
	
	removeAt : function(key, uri){
		var branch = Prefs.getBranch(key);
		for(var i = 0, l = branch.length ;i < l; i++){ //TODO: performance
			var number = branch[i];
			var str = Prefs.getChar(key + number);
			var json = JSONUtils.fromJSON(str);
			
			if(json.uri == uri){
				Prefs.remove(key + number);
				return true;
			}
		}
		return false;
	},
	
	clear : function(){
		var inst = Prefs.instance();
		while(!this.isEmpty())
			inst.clearUserPref(Prefs.root + '.' + this.key + this.pop());
	},
	
	clearAll : function(){
		Prefs.getBranch(String.empty).clear();
	},
	
	getBranch : function(key){
		var branch = Prefs.service().getBranch(Prefs.root + '.' + key).getChildList(String.empty, {});
		branch.toHash = Prefs.toHash;
		branch.clear = Prefs.clear;
		branch.key = key;
		return branch;
	},
	
	toHash : function(){
		var inst = Prefs.instance();
		var hash = new ExtendedHash(), k = this.key;
		this.each(function(elem){
			var _key = Prefs.root + '.' + k + elem;
			var _value = JSONUtils.fromJSON(inst.getCharPref(_key));
			hash.put(k + elem, _value);
		});
		return hash;
	}

};

function ExtendedHash(){
	var _hash = [];
	var _trash = [];
	var _length = 0;
	
	this.put = function(key, value){
		if(_hash[key]){
			_trash.push(value);
		}else {
			_hash[key] = value;
			_length++;
		}
	};
	
	this.get = function(key){
		return _hash[key];
	};
	
	this.remove = function(key){
		var elem;
		if(this.contains(key)){
			elem = _hash[key];
			delete _hash[key];
			_length--;
		}
		return elem;
	};
	
	this.contains = function(key){
		return typeof _hash[key] != 'undefined';
	};
	
	this.length = function(){
		return _length;
	};
	
	this.isEmpty = function(){
		return _length == 0;
	};
	
	this.shift = function(){
		for(var i in _hash){
			var val;
			if(_length != 0){
				val = _hash[i];
				delete _hash[i];
				_length--;
			}
			return val;
		}
	};
	
	this.each = function(f){
		if(_length != 0){
			for(var i in _hash){
				if(!(i in Array.prototype))
					f(i, _hash[i]);
			}
		}
	};
	
	this.trash = function(){
		return _trash;
	};
}
