/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/*
	Use this object to extract the content of xml/xhtml or html resources. 
	[1] If the given document is well-formed all tags (inclusive script part) and whitespace
	sequences will be removed successfully. Otherwise the parser tries to repair the resource 
	with tidy (see http://tidy.sourceforge.net/).  On success the parser continues with [1], 
	otherwise the input string is returned.
	
	Support following charsets:
	
	utf-8 (Unicode, worldwide)
	utf-16 (Unicode, worldwide)
	iso-8859-1 (Western Europe)
	iso-8859-2 (Central Europe)
	iso-8859-3 (Southern Europe)
	iso-8859-4 (Baltic Rim)
	iso-8859-5 (Cyrillic)
	iso-8859-6-i (Arabic)
	iso-8859-7 (Greek)
	iso-8859-8-i (Hebrew)
	iso-8859-9 (Turkish)
	iso-8859-10 (Latin 6)
	iso-8859-13 (Latin 7)
	iso-8859-14 (Celtic)
	iso-8859-15 (Latin 9)
	us-ascii (basic English)
	euc-jp (Japanese, Unix)
	shift_jis (Japanese, Win/Mac)
	iso-2022-jp (Japanese, email)
	euc-kr (Korean)
	gb2312 (Chinese, simplified)
	gb18030 (Chinese, simplified)
	big5 (Chinese, traditional)
	tis-620 (Thai)
	koi8-r (Russian)
	koi8-u (Ukrainian)
	macintosh (MacRoman)
	windows-1250 (Central Europe)
	windows-1251 (Cyrillic)
	windows-1252 (Western Europe)
	windows-1253 (Greek)
	windows-1254 (Turkish)
	windows-1255 (Hebrew)
	windows-1256 (Arabic)
	windows-1257 (Baltic Rim) 
*/
var Parser = {

	tidy : null,

	init : function(){
		var instTidy = Components.classes["@bookmarktools.mozdev.org/repair/tidy;1"].createInstance();
		this.tidy = instTidy.QueryInterface(Components.interfaces.ITidy);
	},
	
	_init : function(){
		this.tidy = null;
	},

	removeWhiteSpaces : function(content){
		return content.replace(/(\s)+/gi, String.empty);
	},
	
	removeNumbers : function(content){
		return content.replace(/[0-9]+/gi, String.empty);
	},
	
	replaceWhiteSpaces : function(content){
		return content.replace(/(\s)+/gi," ");
	},
	
	removeRTFTags : function(content){
		content = content.replace(/\{?\\[a-zA-Z0-9*-]+\s*/gi, String.empty);
		content = content.replace(/\s*[a-zA-Z\s]+;\}/gi, String.empty);
		return content.replaces('}', String.empty);
	},
	
	extractEntities : function(markup){
		return markup.match(/&([a-zA-Z0-9#]+);/gi);
	},
	
	decodeISOLatin1 : function(token){
		//check entity names
		switch(token){
			case '&quot;' : return 34;
			case '&apos;' : return 39;
			case '&amp;' : return 38;
			case '&lt;' : return 60;
			case '&gt;' : return 62;
			case '&nbsp;' : return 160;
			case '&iexcl;' : return 161;
			case '&cent;' : return 162;
			case '&pound;' : return 163;
			case '&curren;' : return 164;
			case '&yen;' : return 165;
			case '&brvbar;' : return 166;
			case '&sect;' : return 167;
			case '&uml;' : return 168;
			case '&copy;' : return 169;
			case '&ordf;' : return 170;
			case '&laquo;' : return 171;
			case '&not;' : return 172;
			case '&shy;' : return 173;
			case '&reg;' : return 174;
			case '&macr;' : return 175;
			case '&deg;' : return 176;
			case '&plusmn;' : return 177;
			case '&sup2;' : return 178;
			case '&sup3;' : return 179;
			case '&acute;' : return 180;
			case '&micro;' : return 181;
			case '&para;' : return 182;
			case '&middot;' : return 183;
			case '&cedil;' : return 184;
			case '&sup1;' : return 185;
			case '&ordm;' : return 186;
			case '&raquo;' : return 187;
			case '&frac14;' : return 188;
			case '&frac12;' : return 189;
			case '&frac34;' : return 190;
			case '&iquest;' : return 191;
			case '&times;' : return 215;
			case '&divide;' : return 247;
			case '&Agrave;' : return 192;
			case '&Aacute;' : return 193;
			case '&Acirc;' : return 194;
			case '&Atilde;' : return 195;
			case '&Auml;' : return 196;
			case '&Aring;' : return 197;
			case '&AElig;' : return 198;
			case '&Ccedil;' : return 199;
			case '&Egrave;' : return 200;
			case '&Eacute;' : return 201;
			case '&Ecirc;' : return 202;
			case '&Euml;' : return 203;
			case '&Igrave;' : return 204;
			case '&Iacute;' : return 205;
			case '&Icirc;' : return 206;
			case '&Iuml;' : return 207;
			case '&ETH;' : return 208;
			case '&Ntilde;' : return 209;
			case '&Ograve;' : return 210;
			case '&Oacute;' : return 211;
			case '&Ocirc;' : return 212;
			case '&Otilde;' : return 213;
			case '&Ouml;' : return 214;
			case '&Oslash;' : return 216;
			case '&Ugrave;' : return 217;
			case '&Uacute;' : return 218;
			case '&Ucirc;' : return 219;
			case '&Uuml;' : return 220;
			case '&Yacute;' : return 221;
			case '&THORN;' : return 222;
			case '&szlig;' : return 223;
			case '&agrave;' : return 224;
			case '&aacute;' : return 225;
			case '&acirc;' : return 226;
			case '&atilde;' : return 227;
			case '&auml;' : return 228;
			case '&aring;' : return 229;
			case '&aelig;' : return 230;
			case '&ccedil;' : return 231;
			case '&egrave;' : return 232;
			case '&eacute;' : return 233;
			case '&ecirc;' : return 234;
			case '&euml;' : return 235;
			case '&igrave;' : return 236;
			case '&iacute;' : return 237;
			case '&icirc;' : return 238;
			case '&iuml;' : return 239;
			case '&eth;' : return 240;
			case '&ntilde;' : return 241;
			case '&ograve;' : return 242;
			case '&oacute;' : return 243;
			case '&ocirc;' : return 244;
			case '&otilde;' : return 245;
			case '&ouml;' : return 246;
			case '&oslash;' : return 248;
			case '&ugrave;' : return 249;
			case '&uacute;' : return 250;
			case '&ucirc;' : return 251;
			case '&uuml;' : return 252;
			case '&yacute;' : return 253;
			case '&thorn;' : return 254;
			case '&yuml;' : return 255;
			default: break;
		}
		
		//check entity numbers
		switch(token){
			case '&#34;' : return 34;
			case '&#39;' : return 39;
			case '&#38;' : return 38;
			case '&#60;' : return 60;
			case '&#62;' : return 62;
			case '&#160;' : return 160;
			case '&#161;' : return 161;
			case '&#162;' : return 162;
			case '&#163;' : return 163;
			case '&#164;' : return 164;
			case '&#165;' : return 165;
			case '&#166;' : return 166;
			case '&#167;' : return 167;
			case '&#168;' : return 168;
			case '&#169;' : return 169;
			case '&#170;' : return 170;
			case '&#171;' : return 171;
			case '&#172;' : return 172;
			case '&#173;' : return 173;
			case '&#174;' : return 174;
			case '&#175;' : return 175;
			case '&#176;' : return 176;
			case '&#177;' : return 177;
			case '&#178;' : return 178;
			case '&#179;' : return 179;
			case '&#180;' : return 180;
			case '&#181;' : return 181;
			case '&#182;' : return 182;
			case '&#183;' : return 183;
			case '&#184;' : return 184;
			case '&#185;' : return 185;
			case '&#186;' : return 186;
			case '&#187;' : return 187;
			case '&#188;' : return 188;
			case '&#189;' : return 189;
			case '&#190;' : return 190;
			case '&#191;' : return 191;
			case '&#215;' : return 215;
			case '&#247;' : return 247;
			case '&#192;' : return 192;
			case '&#193;' : return 193;
			case '&#194;' : return 194;
			case '&#195;' : return 195;
			case '&#196;' : return 196;
			case '&#197;' : return 197;
			case '&#198;' : return 198;
			case '&#199;' : return 199;
			case '&#200;' : return 200;
			case '&#201;' : return 201;
			case '&#202;' : return 202;
			case '&#203;' : return 203;
			case '&#204;' : return 204;
			case '&#205;' : return 205;
			case '&#206;' : return 206;
			case '&#207;' : return 207;
			case '&#208;' : return 208;
			case '&#209;' : return 209;
			case '&#210;' : return 210;
			case '&#211;' : return 211;
			case '&#212;' : return 212;
			case '&#213;' : return 213;
			case '&#214;' : return 214;
			case '&#216;' : return 216;
			case '&#217;' : return 217;
			case '&#218;' : return 218;
			case '&#219;' : return 219;
			case '&#220;' : return 220;
			case '&#221;' : return 221;
			case '&#222;' : return 222;
			case '&#223;' : return 223;
			case '&#224;' : return 224;
			case '&#225;' : return 225;
			case '&#226;' : return 226;
			case '&#227;' : return 227;
			case '&#228;' : return 228;
			case '&#229;' : return 229;
			case '&#230;' : return 230;
			case '&#231;' : return 231;
			case '&#232;' : return 232;
			case '&#233;' : return 233;
			case '&#234;' : return 234;
			case '&#235;' : return 235;
			case '&#236;' : return 236;
			case '&#237;' : return 237;
			case '&#238;' : return 238;
			case '&#239;' : return 239;
			case '&#240;' : return 240;
			case '&#241;' : return 241;
			case '&#242;' : return 242;
			case '&#243;' : return 243;
			case '&#244;' : return 244;
			case '&#245;' : return 245;
			case '&#246;' : return 246;
			case '&#248;' : return 248;
			case '&#249;' : return 249;
			case '&#250;' : return 250;
			case '&#251;' : return 251;
			case '&#252;' : return 252;
			case '&#253;' : return 253;
			case '&#254;' : return 254;
			case '&#255;' : return 255;
			default: break;
		}

		return -1;
	},
	
	filterISOLatin1 : function(charCode){
		switch (charCode) {
			case 0xC0 : // 
			case 0xC1 : // 
			case 0xC2 : // 
			case 0xC3 : // 
			case 0xC4 : // 
			case 0xC5 : // 
				return 'A';
			case 0xC6 : // 
				return "AE";
			case 0xC7 : // 
				return 'C';
			case 0xC8 : // 
			case 0xC9 : // 
			case 0xCA : // 
			case 0xCB : // 
				return 'E';
			case 0xCC : // 
			case 0xCD : // 
			case 0xCE : // 
			case 0xCF : // 
				return 'I';
			case 0xD0 : // 
				return 'D';
			case 0xD1 : // 
				return 'N';
			case 0xD2 : // 
			case 0xD3 : // 
			case 0xD4 : // 
			case 0xD5 : // 
			case 0xD6 : // 
			case 0xD8 : // 
				return 'O';
			case 0xDE : // 
				return "TH";
			case 0xD9 : // 
			case 0xDA : // 
			case 0xDB : // 
			case 0xDC : // 
				return 'U';
			case 0xDD : // 
				return 'Y';
			case 0xE0 : // 
			case 0xE1 : // 
			case 0xE2 : // 
			case 0xE3 : // 
			case 0xE4 : // 
			case 0xE5 : // 
				return 'a';
			case 0xE6 : // 
				return "ae";
			case 0xE7 : // 
				return 'c';
			case 0xE8 : // 
			case 0xE9 : // 
			case 0xEA : // 
			case 0xEB : // 
				return 'e';
			case 0xEC : // 
			case 0xED : // 
			case 0xEE : // 
			case 0xEF : // 
				return 'i';
			case 0xF0 : // 
				return 'd';
			case 0xF1 : // 
				return 'n';
			case 0xF2 : // 
			case 0xF3 : // 
			case 0xF4 : // 
			case 0xF5 : // 
			case 0xF6 : // 
			case 0xF8 : // 
				return 'o';
			case 0xDF : // 
				return "ss";
			case 0xFE : // 
				return "th";
			case 0xF9 : // 
			case 0xFA : // 
			case 0xFB : // 
			case 0xFC : // 
				return 'u';
			case 0xFD : // 
			case 0xFF : // 
				return 'y';
			default : break;
		}
		return null;
	},
	
	filterISOLatin1Entities : function(content){
		var entities = Parser.extractEntities(content);
		while(entities.length != 0){
			var entity = entities.shift();
			var charCode = Parser.decodeISOLatin1(entity);
			if(charCode != -1){
				var filter = Parser.filterISOLatin1(charCode);
				if(filter != null){
					content = content.replace(entity, filter);
				}else {
					content = content.replace(entity, String.empty);
				}
			}
		}
		return content;
	},
	
	filterISOLatin1Query : function(query){
		for(var i = 0, l = query.length ;i < l; i++){
			var charCode = query.charCodeAt(i);
			if(charCode > 127){ /* 7-bit -> US-ASCII, greater is ISOLatin1 */
				var filter = Parser.filterISOLatin1(query.charCodeAt(i));
				if(filter != null){
					query = query.replace(query.charAt(i), filter);
				}else {
					query = query.replace(query.charAt(i), String.empty);
				}
			}
		}
		return query;
	},
	
	/*parse : function(content, mimeType, charSet){
		switch(mimeType){
		
		}
	},*/
	
	parse : function(content){
		var response = this.tidy.cleanup(content);
		if(response != "Tidy: no output"){
			response = this.replaceWhiteSpaces(response);
			response = response.stripScripts(); /* prototype.js by Sam Stephenson */
			response = response.stripTags(); /* prototype.js by Sam Stephenson */
			return response;
		}

		return this.replaceWhiteSpaces(content);
	}
};